@@ -1,4 +1,3 @@
-import os.path
 #!/usr/bin/python
 #----------------------------------------------------------------------------
 # Sample of a Rana module.
@@ -20,15 +19,15 @@ import os.path
 #---------------------------------------------------------------------------
 #from dbus.service import Object
 from base_module import ranaModule
-from upoints import gpx
 import geo
 import os
+import modrana_utils
 import glob
 import cPickle
-#import marshal
+import shutil
+import modrana_utils
 from time import clock
 from time import gmtime, strftime
-#from time import clock
 
 def getModule(m,d,i):
   return(loadTracklogs(m,d,i))
@@ -40,29 +39,13 @@ class loadTracklogs(ranaModule):
     ranaModule.__init__(self, m, d, i)
     self.tracklogs = {} # dictionary of all loaded tracklogs, path is the key
     #self.set('tracklogs', self.tracklogs) # now we make the list easily acessible to other modules
-    self.cachePath = 'cache/tracklogs/tracklog_cache.txt'
     self.cache = {}
-    self.tracklogFolder = 'tracklogs/'
     self.tracklogList = []
     self.tracklogPathList = []
     self.categoryList = []
 
-
   def firstTime(self):
-    folder = self.get('tracklogFolder', 'tracklogs/')
-
-    if folder != None:
-      
-      # make sure there is a slash on the end
-      # TODO: check if this is too platform specific
-      if folder[-1] == '/':
-        self.tracklogFolder = folder
-      else:
-        self.tracklogFolder = (folder+'/')
-    else:
-      self.tracklogFolder = 'tracklogs/'
-
-#    self.load()
+    self._createBasicFolderStructure()
 
   def handleMessage(self, message, type, args):
     if message == 'loadActive':
@@ -98,6 +81,51 @@ class loadTracklogs(ranaModule):
 #        # get current tracklog filename, sans extension
 #        # start an entry box
 
+  def _getTFP(self):
+    options = self.m.get('options', None)
+    if options:
+      tracklogFolder = options.getTracklogsFolderPath()
+      # check if the folder exists and create it if it doesn't
+      modrana_utils.createFolderPath(tracklogFolder)
+      return tracklogFolder
+    else:
+      print("loadTracklogs:no options module, can't get tracklog folder path")
+      return None
+
+  def _getTFSubPath(self, subPath):
+    """return a tracklog folder sub path,
+    assure the patch exists before returning it"""
+    tracklogFolderPath = self._getTFP()
+    if tracklogFolderPath == None:
+      print("loadTracklogs: can't get tracklog sub path - tracklog folder path is unknown")
+      return None # tracklog folder path is unknown
+    else:
+      TFSubPath = os.path.join(tracklogFolderPath, subPath)
+      modrana_utils.createFolderPath(TFSubPath)
+      return TFSubPath
+
+  def _createBasicFolderStructure(self):
+    """trigger creation of the logs, misc and online folders
+    also copy example tracklogs, if necessary"""
+    self._getTFSubPath("logs")
+    self._getTFSubPath("online")
+    self._getTFSubPath("misc")
+    """if there is no examplef folder, create it
+    and copy example tracklogs into it"""
+    examplesDestinationPath = os.path.join(self._getTFP(),'examples')
+    if not os.path.exists(examplesDestinationPath):
+      modrana_utils.createFolderPath(examplesDestinationPath)
+      print(' ** loadTracklogs: copying example tracklogs')
+      examplesSourcePath = 'data/tracklog_examples'
+       # copy all files from this folder
+      for item in os.listdir(examplesSourcePath):
+        path = os.path.join(examplesSourcePath,item)
+        if os.path.isfile(path):
+          print(' ** copying: %r' % item)
+          shutil.copy(path,os.path.join(examplesDestinationPath,item))
+      print(' ** DONE')
+
+
 
 
   def loadCache(self):
@@ -105,8 +133,9 @@ class loadTracklogs(ranaModule):
     print "loading cache"
     start = clock()
     try:
-      f = open(self.cachePath, 'r')
+      f = open(self.getTracklogCachePath(), 'r')
       self.cache = cPickle.load(f)
+      f.close()
     except:
       print "loadTracklogs: loading cache from file failed"
       self.cache = {}
@@ -198,27 +227,33 @@ class loadTracklogs(ranaModule):
     path = self.get('activeTracklogPath', None)
     return path
 
-
+  def getTracklogCachePath(self):
+    options = self.m.get('options', None)
+    if options:
+      return os.path.join(options.getCacheFolderPath(), 'tracklog_cache.txt')
+    else:
+      print("loadTracklogs: options module missing")
+      return None
 
   def listAvailableTracklogs(self):
     print "** making a list of available tracklogs"
 
-    tf = self.tracklogFolder
+    tf = self._getTFP()
     # does the tracklog folder exist ?
-    if not os.path.exists(tf):
+    if tf == None or not os.path.exists(tf):
       return # no tracklog folder, nothing to list
     # get the available directories,
     # each directory represents a category
     currentFolders = os.listdir(tf)
-    # leave just nonhidden folders
-    currentFolders = filter(lambda x: os.path.isdir(tf+x) and x[0]!='.', currentFolders)
+    # leave just folders
+    currentFolders = filter(lambda x: os.path.isdir(os.path.join(tf, x)), currentFolders)
     # add files from all available folders
     availableFiles = []
     pathList = []
     for folder in currentFolders:
-      #TODO: suport other tracklogs
-      folderFiles = glob.glob(tf+folder+'/*.gpx')
-      folderFiles.extend(glob.glob(tf+folder+'/*.GPX'))
+      #TODO: support other tracklogs
+      folderFiles = glob.glob(os.path.join(tf, folder, '*.gpx'))
+      folderFiles.extend(glob.glob(os.path.join(tf, folder, '*.GPX')))
       # remove possible folders
       folderFiles = filter(lambda x: os.path.isfile(x), folderFiles)
       for file in folderFiles:
@@ -226,7 +261,7 @@ class loadTracklogs(ranaModule):
         filename = os.path.split(file)[1]
         lastModifiedEpochSecs = os.path.getmtime(path)
         lastModified = strftime("%d.%m.%Y %H:%M:%S",gmtime(lastModifiedEpochSecs))
-        size = self.convertBytes(os.path.getsize(path))
+        size = modrana_utils.bytes2PrettyUnitString(os.path.getsize(path))
         extension = os.path.splitext(path)[1]
         cat = folder
         item={'path':path,
@@ -241,11 +276,11 @@ class loadTracklogs(ranaModule):
 
     self.categoryList = currentFolders
 
-    print "*  using this tracklog folder:"
-    print self.tracklogFolder
-    print "*  does it exist ?"
-    print os.path.exists(self.tracklogFolder)
-    print "*  there are %d tracklogs available" % len(availableFiles)
+    print("*  using this tracklog folder:")
+    print(self._getTFP())
+    print("*  does it exist ?")
+    print(os.path.exists(self._getTFP()))
+    print("*  there are %d tracklogs available" % len(availableFiles))
     self.tracklogPathList = pathList
     self.tracklogList = availableFiles
 
@@ -282,25 +317,6 @@ class loadTracklogs(ranaModule):
 #    # update the current in memmory list
 #    self.tracklogList[index]['cat'] = cathegory
 
-  # from:
-  # http://www.5dollarwhitebox.org/drupal/node/84
-  def convertBytes(self, bytes):
-      bytes = float(bytes)
-      if bytes >= 1099511627776:
-          terabytes = bytes / 1099511627776
-          size = '%.2fTB' % terabytes
-      elif bytes >= 1073741824:
-          gigabytes = bytes / 1073741824
-          size = '%.2fGB' % gigabytes
-      elif bytes >= 1048576:
-          megabytes = bytes / 1048576
-          size = '%.2fMB' % megabytes
-      elif bytes >= 1024:
-          kilobytes = bytes / 1024
-          size = '%.2fKB' % kilobytes
-      else:
-          size = '%.2fb' % bytes
-      return size
 
 #  def load(self):
 #    start = clock()
@@ -336,13 +352,13 @@ class loadTracklogs(ranaModule):
 
   def save(self):
     try:
-      f = open(self.cachePath, 'w')
+      print self.getTracklogCachePath()
+      f = open(self.getTracklogCachePath(), 'w')
       cPickle.dump(self.cache, f)
       f.close()
-    except:
-      print "loadTracklogs: cant store tracklog data to cache, tracklogs wil be loaded from files next time"
-
-#  def saveClusters(self, clusters):
+    except Exception, e:
+      print "loadTracklogs: can't store tracklog data to cache, tracklogs will be loaded from files next time"
+      print("exception: %r" % e)
 
   def setPathAsActiveTracklog(self, path):
     self.set('activeTracklogPath', path)
@@ -390,8 +406,9 @@ class loadTracklogs(ranaModule):
       self.sendMessage('notification:loading %s#1' % path)
 
     if(file): # TODO: add handling of other than GPX files
+      # import the GPX module only when really needed
+      from upoints import gpx
       track = gpx.Trackpoints() # create new Trackpoints object
-#      print track
       # lets assume we have only GPX 1.1 files TODO: 1.1 and 1.0
       try:
         track.import_locations(file, "1.1") # load a gpx file into it
@@ -400,8 +417,6 @@ class loadTracklogs(ranaModule):
         if notify:
           self.sendMessage('notification:loading tracklog failed#2')
         return
-#      print file
-#      print track
       file.close()
 
       type="GPX" #TODO: more formats support
@@ -424,6 +439,8 @@ class loadTracklogs(ranaModule):
        then load this file to tracklogs list,
        return resulting path
        or None when storing fails"""
+    # import the GPX module only when really needed
+    from upoints import gpx
     newTracklog = gpx.Trackpoints()
     trackpoints = map(lambda x: gpx.Trackpoint(x[0],x[1]), route)
     newTracklog.append(trackpoints)
@@ -439,28 +456,21 @@ class loadTracklogs(ranaModule):
 
   def storeTracklog(self,tracklog,filename,cat,type,refresh="True"):
     """store tracklog and return the resulting path"""
-    folder = self.tracklogFolder
-    path = folder + cat +'/'
-    # does the directory exist ?
-    if not os.path.exists(path):
-      # try to create the directory
-      try:
-        os.makedirs(path)
-      except Exception, e:
-        print "loadTracklogs: tracklog folder unusable"
-        print "exception: %s" % e
-        self.sendMessage('notification:Error: tracks folder unusable#3')
-        return None
+    folder = self._getTFP()
+    if folder == None:
+      print("loadTracklogs: can't store tracklog - path to tracklog folder is unknown or unusable")
+      return None
+    path = os.path.join(folder, cat)
     # is it a directory ?
     if not os.path.isdir(path):
-      self.sendMessage('Error: tracks folder unusable#3')
+      self.sendMessage("loadTracklogs: can't store tracklog - tracklgo folder is not a directory")
       return None
 
     if type=="GPX":
       # try to create the file
       try:
         xmlTree = tracklog.export_gpx_file()
-        f = open(path+filename, 'w')
+        f = open(os.path.join(path,filename), 'w')
         xmlTree.write(f)
         f.close()
       except Exception, e:
@@ -484,11 +494,6 @@ class loadTracklogs(ranaModule):
       dy = y2 - y1
       return math.sqrt(dx**2 + dy**2)
 
-#  def adjustAproximateCircle(self, points, radius):
-#    """test wheter all points are inside the preliminary circle,
-#    if not, adjust the radius acordingly"""
-##    for point in points:
-
   # found on:
   # http://www.quanative.com/2010/01/01/server-side-marker-clustering-for-google-maps-with-python/
   def clusterTrackpoints(self, trackpointsList , cluster_distance):
@@ -620,12 +625,6 @@ class GPXTracklog(tracklog):
       cache[filename] = ci
       
 
-#    self.checkElevation()
-#
-#    if self.elevation == True:
-#      self.getPerElev()
-
-
   def modified(self):
     """the tracklog has been modified, recount all the statistics and clusters"""
     # TODO: implement this ? :D
@@ -682,13 +681,6 @@ class GPXTracklog(tracklog):
   def getPerElev(self):
     self.perElevList = geo.perElevList(self.trackpointsList)
 
-#  def getLength(self):
-#    """return length of the tracklog if known, None else - GPX version"""
-#    if self.perElevList:
-#      print self.perElevList
-#    else:
-#      return None
-
 class CacheItem():
   """class representing a cache item"""
   def __init__(self, clusters, routeInfo=None, perElevList=None):
